@@ -9,22 +9,109 @@ import shutil
 import textwrap
 from typing import Any, Dict, List, Optional, Union
 import tempfile
-from subprocess import run as sp_run
+from subprocess import CalledProcessError, run as sp_run
 from glob import glob
 
 from .config import Config
 
+from distlib.version import NormalizedVersion
+
+_CMAKE_MINIMUM_REQUIRED = NormalizedVersion('3.15')
+
 
 class _BuildBackend(object):
 
     def __init__(self) -> None:
         self.verbose = False
 
-    fileopt = {'encoding': 'utf-8'}
+    def check_program_version(
+        self,
+        program: str,
+        minimum_version: Optional[NormalizedVersion],
+        name: Optional[str],
+        check_version: bool = True,
+    ):
+        """Check if there's a new enough version of the given command available
+        in PATH."""
+        name = name or program
+        try:
+            # Try running the command
+            cmd = [program, '--version'] if check_version else [program, '-h']
+            res = self.run(cmd, check=True, capture_output=True)
+            # Try finding the version
+            if check_version:
+                m = re.search(r'\d+(\.\d+){1,}', res.stdout.decode('utf-8'))
+                if not m:
+                    raise RuntimeError(f"Unexpected {name} version output")
+                program_version = NormalizedVersion(m.group(0))
+                if self.verbose: print("Found", name, program_version)
+                # Check if the version is new enough
+                if minimum_version is not None:
+                    if program_version < minimum_version:
+                        raise RuntimeError(f"{name} too old")
+        except CalledProcessError as e:
+            if self.verbose:
+                print(f'{type(e).__module__}.{type(e).__name__}', e, sep=': ')
+                sys.stdout.buffer.write(e.stdout)
+                sys.stdout.buffer.write(e.stderr)
+            return False
+        except Exception as e:
+            # If any of that failed, return False
+            if self.verbose:
+                print(f'{type(e).__module__}.{type(e).__name__}', e, sep=': ')
+            return False
+        return True
 
     def get_requires_for_build_wheel(self, config_settings=None):
         """https://www.python.org/dev/peps/pep-0517/#get-requires-for-build-wheel"""
-        return []
+        self.parse_config_settings(config_settings)
+        pyproject = Path('pyproject.toml').resolve()
+        cfg = self.read_metadata(pyproject)
+        deps = []
+        # Check if we need CMake
+        if cfg.cmake:
+            self.check_cmake_program(cfg, deps)
+        if cfg.stubgen:
+            self.check_stubgen_program(cfg, deps)
+        if self.verbose:
+            print('Dependencies for build:', deps)
+        return deps
+
+    def check_cmake_program(self, cfg: Config, deps: List[str]):
+        assert cfg.cmake
+        # Do we need to perform a native build?
+        native = not cfg.cross or self.needs_cross_native_build(cfg)
+        native_cfg = cfg.cmake[self.get_os_name()] if native else {}
+        # Do we need to perform a cross build?
+        cross = cfg.cross
+        cross_cfg = cfg.cmake.get('cross', {})
+        # Find the strictest version requirement
+        min_cmake_ver = max(
+            _CMAKE_MINIMUM_REQUIRED,
+            NormalizedVersion(native_cfg.get('minimum_version', '0.0')),
+            NormalizedVersion(cross_cfg.get('minimum_version', '0.0')),
+        )
+        # If CMake in PATH doesn't work or is too old, add it as a build
+        # requirement
+        if not self.check_program_version('cmake', min_cmake_ver, "CMake"):
+            deps.append("cmake>=" + str(min_cmake_ver))
+
+        # Check if we need Ninja
+        cfgs = []
+        if native: cfgs.append(native_cfg)
+        if cross: cfgs.append(cross_cfg)
+        # Do any of the configs require Ninja as a generator?
+        needs_ninja = lambda c: 'ninja' in c.get('generator', '').lower()
+        need_ninja = any(map(needs_ninja, cfgs))
+        if need_ninja:
+            # If so, check if a working version exists in the PATH, otherwise,
+            # add it as a build requirement
+            if not self.check_program_version('ninja', None, "Ninja"):
+                deps.append("ninja")
+
+    def check_stubgen_program(self, cfg: Config, deps: List[str]):
+        if not self.check_program_version('stubgen', None, None, False):
+            deps.append("mypy")
 
     def get_requires_for_build_editable(self, config_settings=None):
         """https://www.python.org/dev/peps/pep-0660/#get-requires-for-build-editable"""
@@ -32,7 +119,7 @@ class _BuildBackend(object):
 
     def get_requires_for_build_sdist(self, config_settings=None):
         """https://www.python.org/dev/peps/pep-0517/#get-requires-for-build-sdist"""
-        return self.get_requires_for_build_wheel(config_settings)
+        return []
 
     def build_wheel(self,
                     wheel_directory,
@@ -79,8 +166,8 @@ class _BuildBackend(object):
         from flit_core.common import Module, make_metadata
         pyproject = src_dir / 'pyproject.toml'
         cfg = self.read_metadata(pyproject)
-        norm_name = normalize_name_wheel(cfg.metadata['name'])
-        pkg = Module(norm_name, src_dir / cfg.module.get('directory', '.'))
+        import_name = cfg.module['name']
+        pkg = Module(import_name, src_dir / cfg.module['directory'])
         metadata = make_metadata(pkg, cfg)
         metadata.version = self.normalize_version(metadata.version)
 
@@ -112,6 +199,8 @@ class _BuildBackend(object):
         }[platform.system()]
 
     def parse_config_settings(self, config_settings: Optional[Dict]):
+        if 'PY_BUILD_CMAKE_VERBOSE' in os.environ:
+            self.verbose = True
         if config_settings is None:
             return
         if config_settings.keys() & {'verbose', 'v'}:
@@ -119,7 +208,18 @@ class _BuildBackend(object):
 
     def read_metadata(self, pyproject):
         from .config import read_metadata
-        cfg = read_metadata(pyproject)
+        from flit_core.config import ConfigError
+        try:
+            cfg = read_metadata(pyproject)
+        except ConfigError as e:
+            if not self.verbose:
+                e.args = ("\n"
+                        "\n"
+                        "\t❌ Error in configuration:\n"
+                        "\n"
+                        f"\t\t{e}\n"
+                        "\n", )
+            raise
         if self.verbose:
             print("\npy-build-cmake options")
             print("======================")
@@ -149,8 +249,9 @@ class _BuildBackend(object):
         from .config import normalize_name_wheel
         from flit_core.common import Module, make_metadata
         cfg = self.read_metadata(src_dir / 'pyproject.toml')
-        norm_name = normalize_name_wheel(cfg.metadata['name'])
-        pkg = Module(norm_name, src_dir / cfg.module.get('directory', '.'))
+        dist_name = normalize_name_wheel(cfg.metadata['name'])
+        import_name = cfg.module['name']
+        pkg = Module(import_name, src_dir / cfg.module['directory'])
         metadata = make_metadata(pkg, cfg)
         metadata.version = self.normalize_version(metadata.version)
 
@@ -161,19 +262,19 @@ class _BuildBackend(object):
             self.write_editable_wrapper(staging_dir, src_dir, pkg)
 
         # Create dist-info folder
-        distinfo = staging_dir / f'{norm_name}-{metadata.version}.dist-info'
+        distinfo = staging_dir / f'{dist_name}-{metadata.version}.dist-info'
         os.makedirs(distinfo, exist_ok=True)
 
         # Write metadata
         metadata_path = distinfo / 'METADATA'
-        with open(metadata_path, 'w', **self.fileopt) as f:
+        with open(metadata_path, 'w', encoding='utf-8') as f:
             metadata.write_metadata_file(f)
 
         # Write or copy license
         self.write_license_files(cfg.license, src_dir, distinfo)
 
-        # TODO: write entrypoints
-        self.write_entrypoints(cfg)
+        # Write entrypoints
+        self.write_entrypoints(distinfo, cfg)
 
         # Generate .pyi stubs
         if cfg.stubgen is not None and not editable:
@@ -182,15 +283,19 @@ class _BuildBackend(object):
         # Configure, build and install the CMake project
         if cfg.cmake:
             self.do_native_cross_cmake_build(tmp_build_dir, staging_dir,
-                                             src_dir, cfg, metadata)
+                                             src_dir, cfg, metadata, dist_name,
+                                             import_name)
 
         # Create wheel
         whl_name = self.create_wheel(wheel_directory, staging_dir, cfg,
-                                     norm_name, metadata.version)
+                                     dist_name, metadata.version)
         return whl_name
 
+    def needs_cross_native_build(self, cfg):
+        return cfg.cross and 'copy_from_native_build' in cfg.cross
+
     def do_native_cross_cmake_build(self, tmp_build_dir, staging_dir, src_dir,
-                                    cfg, metadata):
+                                    cfg, metadata, dist_name, import_name):
         """If not cross-compiling, just do a regular CMake build+install.
         When cross-compiling, do a cross-build+install (using the provided 
         CMake toolchain file).
@@ -203,18 +308,19 @@ class _BuildBackend(object):
         # When cross-compiling, optionally do a native build first
         native_install_dir = None
         native_cmake_cfg = cfg.cmake[self.get_os_name()]
-        if cfg.cross and (cfnb := 'copy_from_native_build') in cfg.cross:
+        if self.needs_cross_native_build(cfg):
             native_install_dir = tmp_build_dir / 'native-install'
             self.run_cmake(src_dir, native_install_dir, metadata,
-                           native_cmake_cfg, None, native_install_dir)
+                           native_cmake_cfg, None, native_install_dir,
+                           dist_name, import_name)
         # Then do the actual build
         cmake_cfg = cfg.cmake['cross'] if cfg.cross else native_cmake_cfg
         self.run_cmake(src_dir, staging_dir, metadata, cmake_cfg, cfg.cross,
-                       native_install_dir)
+                       native_install_dir, dist_name, import_name)
         # Finally, move the files from the native build to the staging area
         if native_install_dir:
             self.copy_native_install(staging_dir, native_install_dir,
-                                     cfg.cross[cfnb])
+                                     cfg.cross['copy_from_native_build'])
 
     def copy_native_install(self, staging_dir, native_install_dir,
                             native_install_patterns):
@@ -242,14 +348,16 @@ class _BuildBackend(object):
 
     def write_license_files(self, license, srcdir: Path, distinfo_dir: Path):
         if 'text' in license:
-            with open(distinfo_dir / 'LICENSE', 'w', encoding='utf-8') as f:
+            with (distinfo_dir / 'LICENSE').open('w', encoding='utf-8') as f:
                 f.write(license['text'])
         elif 'file' in license:
             shutil.copy2(srcdir / license['file'], distinfo_dir)
 
     def write_editable_wrapper(self, tmp_build_dir: Path, src_dir: Path, pkg):
         # Write a fake __init__.py file that points to the development folder
-        tmp_pkg = tmp_build_dir / pkg.name
+        tmp_pkg: Path = tmp_build_dir / pkg.name
+        pkgpath = Path(pkg.path)
+        initpath = pkgpath / '__init__.py'
         os.makedirs(tmp_pkg, exist_ok=True)
         special_dunders = [
             '__builtins__', '__cached__', '__file__', '__loader__', '__name__',
@@ -257,25 +365,22 @@ class _BuildBackend(object):
         ]
         content = f"""\
             # First extend the search path with the development folder
-            __spec__.submodule_search_locations.insert(0, "{pkg.path}")
+            __spec__.submodule_search_locations.insert(0, {str(pkgpath)!a})
             # Now manually import the development __init__.py
             from importlib import util as _util
             _spec = _util.spec_from_file_location("{pkg.name}",
-                                                  "{pkg.path}/__init__.py")
+                                                  {str(initpath)!a})
             _mod = _util.module_from_spec(_spec)
             _spec.loader.exec_module(_mod)
             # After importing, add its symbols to our global scope
             _vars = _mod.__dict__.copy()
             for _k in ['{"','".join(special_dunders)}']: _vars.pop(_k)
             globals().update(_vars)
-            del _k
-            del _spec
-            del _mod
-            del _vars
-            del _util
+            # Clean up
+            del _k, _spec, _mod, _vars, _util
             """
         (tmp_pkg / '__init__.py').write_text(textwrap.dedent(content),
-                                             **self.fileopt)
+                                             encoding='utf-8')
         # Add the py.typed file if it exists, so mypy picks up the stubs for
         # the C++ extensions
         py_typed: Path = pkg.path / 'py.typed'
@@ -292,7 +397,7 @@ class _BuildBackend(object):
             shutil.copy2(mod_file, dst, follow_symlinks=False)
 
     def run(self, *args, **kwargs):
-        """Wrapper around subproces.run that optionally prints the command."""
+        """Wrapper around subprocess.run that optionally prints the command."""
         if self.verbose:
             pprint([*args])
             pprint(kwargs)
@@ -311,19 +416,14 @@ class _BuildBackend(object):
         args += cfg.get('files', [])
         # Add output folder argument if not already specified in cfg['args']
         if 'args' not in cfg or not ({'-o', '--output'} & set(cfg['args'])):
-            args += ['-o', staging_dir]
+            args += ['-o', str(staging_dir)]
         env = os.environ.copy()
         env.setdefault('MYPY_CACHE_DIR', str(tmp_build_dir))
         # Call mypy stubgen in a subprocess
         self.run(args, cwd=pkg.path.parent, check=True, env=env)
 
-    def run_cmake(self,
-                  pkgdir,
-                  install_dir,
-                  metadata,
-                  cmake_cfg,
-                  cross_cfg,
-                  native_install_dir=None):
+    def run_cmake(self, pkgdir, install_dir, metadata, cmake_cfg, cross_cfg,
+                  native_install_dir, dist_name, import_name):
         """Configure, build and install using CMake."""
         # Source and build folders
         srcdir = Path(cmake_cfg.get('source_path', pkgdir)).resolve()
@@ -336,7 +436,8 @@ class _BuildBackend(object):
         builddir = builddir.resolve()
         # Environment variables
         cmake_env = os.environ.copy()
-        if (f := 'env') in cmake_cfg:
+        f = 'env'
+        if f in cmake_cfg:
             for k, v in cmake_cfg[f].items():
                 cmake_env[k] = Template(v).substitute(cmake_env)
 
@@ -348,7 +449,11 @@ class _BuildBackend(object):
             '-S',
             str(srcdir),
             '-D',
-            'VERIFY_VERSION=' + metadata.version,
+            'PY_BUILD_CMAKE_PACKAGE_VERSION:STRING=' + metadata.version,
+            '-D',
+            'PY_BUILD_CMAKE_PACKAGE_NAME:STRING=' + dist_name,
+            '-D',
+            'PY_BUILD_CMAKE_MODULE_NAME:STRING=' + import_name,
         ]
         if cross_cfg:
             toolchain = (pkgdir / cross_cfg['toolchain_file']).resolve()
@@ -371,9 +476,11 @@ class _BuildBackend(object):
         configure_cmd += cmake_cfg.get('args', [])  # User-supplied arguments
         for k, v in cmake_cfg.get('options', {}).items():  # -D {option}={val}
             configure_cmd += ['-D', k + '=' + v]
-        if btype := cmake_cfg.get('build_type'):  # -D CMAKE_BUILD_TYPE={type}
+        btype = cmake_cfg.get('build_type')
+        if btype:  # -D CMAKE_BUILD_TYPE={type}
             configure_cmd += ['-D', 'CMAKE_BUILD_TYPE=' + btype]
-        if gen := cmake_cfg.get('generator'):  # -G {generator}
+        gen = cmake_cfg.get('generator')
+        if gen:  # -G {generator}
             configure_cmd += ['-G', gen]
         self.run(configure_cmd, check=True, env=cmake_env)
 
@@ -412,7 +519,8 @@ class _BuildBackend(object):
         build_cmd += cmake_cfg.get('build_args', [])  # User-supplied arguments
         if config:  # --config {config}
             build_cmd += ['--config', config]
-        if (f := 'build_tool_args') in cmake_cfg:
+        f = 'build_tool_args'
+        if f in cmake_cfg:
             build_cmd += ['--'] + cmake_cfg[f]
         self.run(build_cmd, check=True, env=cmake_env)
 
@@ -426,23 +534,26 @@ class _BuildBackend(object):
                     _BuildBackend.get_cross_tags(cross_cfg).values()))
         return buildconfig
 
-    def write_entrypoints(self, cfg: Config):
-        if cfg.entrypoints:
-            print(cfg.entrypoints)
-            raise RuntimeWarning("Entrypoints are not currently supported")
+    def write_entrypoints(self, distinfo: Path, cfg: Config):
+        from flit_core.common import write_entry_points
+        with (distinfo / 'entry_points.txt').open('w', encoding='utf-8') as f:
+            write_entry_points(cfg.entrypoints, f)
 
-    def create_wheel(self, wheel_directory, tmp_build_dir, cfg, norm_name,
+    def create_wheel(self, wheel_directory, tmp_build_dir, cfg, dist_name,
                      norm_version):
         from distlib.wheel import Wheel
         whl = Wheel()
-        whl.name = norm_name
+        whl.name = dist_name
         whl.version = norm_version
-        libdir = 'platlib' if cfg.cmake else 'purelib'
+        pure = not cfg.cmake
+        libdir = 'purelib' if pure else 'platlib'
         paths = {'prefix': str(tmp_build_dir), libdir: str(tmp_build_dir)}
         whl.dirname = wheel_directory
         tags = None
         if cfg.cross:
             tags = self.get_cross_tags(cfg.cross)
+        if pure:
+            tags = {'pyver': ['py3']}
         wheel_path = whl.build(paths, tags=tags, wheel_version=(1, 0))
         whl_name = os.path.relpath(wheel_path, wheel_directory)
         return whl_name
